close all;
clear all;

Tmax = 600;
deltaT = 0.1;
T = 0:deltaT:Tmax;
timeSteps = length(T);

% #### Constants #####
CaO2 = 20; % ml O2 / 100 ml blood;
muscleMaxO2uptake = 2*850; % mL O2 / min  for 2 legs
muscleMaxO2storage = 200;
bloodMaxO2uptake = 1000; % mL O2 / min
basalO2consumption = 250; % mL O2 / min
heartStrokeVolume = 70; % mL blood / stroke;
heartRateMax = 220;
heartRateRest = 40;


muscleOutputPower = 1.5*(1-exp(-(1/Tmax)*T))+0.3; % Relative
plot(T,muscleOutputPower,'LineWidth',2);
title('muscleOutputPower');
saveas(gcf,'muscleOutputPower.png','png');

muscleO2storage = zeros(timeSteps,1);
muscleAcid = zeros(timeSteps,1);
muscleGlycose = zeros(timeSteps,1);
muscleO2storage(1) = muscleMaxO2storage;  % mL O2
muscleAcid(1) = 0;  % Relative
muscleGlycose(1) = 1;  % Relative

heartRate = zeros(timeSteps,1);
respiratoryRate = zeros(timeSteps,1);
heartRate(1) = 100;  % Absolute
respiratoryRate(1) = 8;  % Absolute

CvO2 = zeros(timeSteps,1);
CvO2(1) = 15; % ml O2 / 100 ml blood; down to 4 for intensive exercise

cardialOutput = zeros(timeSteps,1);
muscleO2consumption = zeros(timeSteps,1);
muscleO2available = zeros(timeSteps,1);
freeO2afterBasal = zeros(timeSteps,1);

for i=2:timeSteps
    t = T(i);
    
    muscleO2consumption(i) = (muscleMaxO2uptake/60)*muscleOutputPower(i)*deltaT;
    cardialOutput(i) = (heartRate(i-1)/60) * deltaT * heartStrokeVolume;
    cardialO2Output = min(cardialOutput(i) * (CaO2/100),(14*bloodMaxO2uptake/60)*deltaT);
    CurrentBasalO2consumption = (basalO2consumption/60)*deltaT;
    CvO2(i) = cardialO2Output - CurrentBasalO2consumption;
    freeO2afterBasal(i) = cardialO2Output*0.8-CurrentBasalO2consumption;
    muscleO2available(i) = min((muscleMaxO2uptake/60)*deltaT,freeO2afterBasal(i)) ...
                            + muscleO2storage(i-1);
    
    diffMuscleO2storageConsumption = muscleO2storage(i-1) - muscleO2consumption(i);
    if(diffMuscleO2storageConsumption < 0)
        muscleO2storage(i) = 0;
        muscleO2available(i) = muscleO2available(i) - muscleO2storage(i-1);
        diffMuscleO2availableConsumption = muscleO2available(i) - muscleO2consumption(i);
        if(diffMuscleO2availableConsumption < 0)
            CvO2(i) = CvO2(i) - max(0,muscleO2available(i));
            muscleAcid(i) = muscleAcid(i-1) + 0.01*deltaT*-diffMuscleO2availableConsumption;
        else
            O2toCell = min(muscleMaxO2storage,diffMuscleO2availableConsumption);
            muscleO2storage(i) = O2toCell;
            CvO2(i) = CvO2(i) - O2toCell;
        end
    else
        muscleO2storage(i) = diffMuscleO2storageConsumption;
        muscleO2available(i) = muscleO2available(i) - muscleO2storage(i-1);
        O2toCell = muscleMaxO2storage - muscleO2storage(i);
        if(muscleO2available(i) > O2toCell)
            muscleO2storage(i) = muscleMaxO2storage;
            CvO2(i) = CvO2(i) - O2toCell;
        else
            muscleO2storage(i) = muscleO2storage(i) + max(0,muscleO2available(i));
            CvO2(i) = CvO2(i) - max(0,muscleO2available(i));
        end
    end
    CvO2(i) = CvO2(i)*100/cardialOutput(i);
    
    
    heartRate(i) = heartRate(i-1) + deltaT*(9-CvO2(i));
    %muscleO2storageLevel = (muscleMaxO2storage - muscleO2storage(i)) / muscleMaxO2storage;
    %heartRate(i) = (heartRateMax-heartRateRest)*muscleOutputPower(i)*muscleO2storageLevel+heartRateRest;
    if(heartRate(i) > heartRateMax)
        heartRate(i) = heartRateMax;
    end
%     if(CvO2(i) < 12 && heartRate(i-1) < heartRateMax)
%         heartRate(i) = heartRate(i-1) + 1*deltaT;
%     elseif(CvO2(i) > 15)
%         heartRate(i) = heartRate(i-1) - 1*deltaT;
%     else
%         heartRate(i) = heartRate(i-1);
%     end
    
end

figure();
plot(T,CvO2,'LineWidth',2);
title('CvO2');
saveas(gcf,'CvO2.png','png');
figure();
plot(T,muscleO2storage,'LineWidth',2);
title('muscleO2storage');
saveas(gcf,'muscleO2storage.png','png');
figure();
plot(T,heartRate,'LineWidth',2);
title('heartRate');
saveas(gcf,'heartRate.png','png');
figure();
plot(T,freeO2afterBasal,'LineWidth',2);
title('freeO2afterBasal');
saveas(gcf,'freeO2afterBasal.png','png');
figure();
plot(T,muscleO2consumption,'LineWidth',2);
title('muscleO2consumption');
saveas(gcf,'muscleO2consumption.png','png');
figure();
plot(T,muscleO2available,'LineWidth',2);
title('muscleO2available');
saveas(gcf,'muscleO2available.png','png');
figure();
plot(T,muscleAcid,'LineWidth',2);
title('muscleAcid');
saveas(gcf,'muscleAcid.png','png');